package org.joget;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.gpedro.integrations.slack.SlackApi;
import net.gpedro.integrations.slack.SlackAttachment;
import net.gpedro.integrations.slack.SlackField;
import net.gpedro.integrations.slack.SlackMessage;
import org.apache.commons.lang.StringEscapeUtils;
import org.joget.apps.app.model.AppDefinition;
import org.joget.apps.app.service.AppPluginUtil;
import org.joget.apps.app.service.AppUtil;
import org.joget.commons.util.LogUtil;
import org.joget.commons.util.PluginThread;
import org.joget.commons.util.StringUtil;
import org.joget.plugin.base.DefaultApplicationPlugin;
import org.joget.plugin.base.PluginWebSupport;
import org.joget.workflow.model.WorkflowAssignment;
import org.joget.workflow.model.WorkflowProcess;
import org.joget.workflow.model.service.WorkflowManager;
import org.joget.workflow.model.service.WorkflowUserManager;
import org.joget.workflow.util.WorkflowUtil;
import org.json.JSONObject;

public class SlackWebhookTool extends DefaultApplicationPlugin implements PluginWebSupport {
    private final static String MESSAGE_PATH = "message/SlackWebhookTool";
    
    private SlackApi api = null;
    
    @Override
    public String getName() {
        return "Slack Webhook Tool";
    }

    @Override
    public String getVersion() {
        return "5.0.0";
    }
    
    @Override
    public String getClassName() {
        return getClass().getName();
    }
    
    @Override
    public String getLabel() {
        //support i18n
        return AppPluginUtil.getMessage("org.joget.SlackWebhookTool.pluginLabel", getClassName(), MESSAGE_PATH);
    }

    @Override
    public String getDescription() {
        //support i18n
        return AppPluginUtil.getMessage("org.joget.SlackWebhookTool.pluginDesc", getClassName(), MESSAGE_PATH);
    }
    
    @Override
    public String getPropertyOptions() {
        return AppUtil.readPluginResource(getClass().getName(), "/properties/slackWebhookTool.json", null, true, MESSAGE_PATH);
    }

    @Override
    public Object execute(Map props) {
        final Set<String> channelList = new HashSet<String>();
        
        String channels = getPropertyString("channels");
        if (!channels.isEmpty()) {
            String[] temp = channels.split(";");
            for (String t : temp) {
                if (t.startsWith("#")) {
                    channelList.add(t);
                }
            }
        }
        
        WorkflowAssignment wfAssignment = (WorkflowAssignment) getProperty("workflowAssignment");
        
        String participantIds = getPropertyString("participantIds");
        if (wfAssignment != null && !participantIds.isEmpty()) {
            String[] temp = participantIds.split(";");
            if (temp.length > 0) {
                WorkflowManager workflowManager = (WorkflowManager) AppUtil.getApplicationContext().getBean("workflowManager");
                WorkflowProcess process = workflowManager.getProcess(wfAssignment.getProcessDefId());
                for (String pid : temp) {
                    pid = pid.trim();
                    if (!pid.isEmpty()) {
                        Collection<String> userList = WorkflowUtil.getAssignmentUsers(process.getPackageId(), wfAssignment.getProcessDefId(), wfAssignment.getProcessId(), wfAssignment.getProcessVersion(), wfAssignment.getActivityId(), "", pid);

                        if (userList != null && userList.size() > 0) {
                            for (String username : userList) {
                                channelList.add(getSlackUsername(username, wfAssignment));
                            }
                        }
                    }
                }
            }
        }
        
        String usernames = getPropertyString("usernames");
        if (!usernames.isEmpty()) {
            String[] temp = usernames.split(";");
            for (String username : temp) {
                channelList.add(getSlackUsername(username, wfAssignment));
            }
        }
        
        Thread sendMessageThread = new PluginThread(new Runnable() {

            public void run() {
                try {
                    LogUtil.info(SlackWebhookTool.class.getName(), "SlackWebhookTool: Sending message to=" + channelList.toString());
                    sendMessage(channelList);
                    LogUtil.info(SlackWebhookTool.class.getName(), "SlackWebhookTool: Sending message completed to=" + channelList.toString());
                } catch (Exception ex) {
                    LogUtil.error(SlackWebhookTool.class.getName(), ex, "");
                }
            }
        });
        
        sendMessageThread.setDaemon(true);
        sendMessageThread.start();
        
        return null;
    }

    public void webService(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        boolean isAdmin = WorkflowUtil.isCurrentUserInRole(WorkflowUserManager.ROLE_ADMIN);
        if (!isAdmin) {
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
            return;
        }
        
        String action = request.getParameter("action");
        if ("sendTestMessage".equals(action)) {
            String message = "";
            try {
                AppDefinition appDef = AppUtil.getCurrentAppDefinition();
                
                String url = AppUtil.processHashVariable(request.getParameter("url"), null, null, null, appDef);
                String testChannel = AppUtil.processHashVariable(request.getParameter("testChannel"), null, null, null, appDef);
                
                setProperty("url", url);
                setProperty("text", AppPluginUtil.getMessage("SlackWebhookTool.testMessage", getClassName(), MESSAGE_PATH));
                
                if (testChannel != null && !testChannel.isEmpty()) {
                    Set<String> channels = new HashSet<String>();
                    channels.add(testChannel);
                    sendMessage(channels);
                } else {
                    sendMessage(null);
                }
                
                message = AppPluginUtil.getMessage("SlackWebhookTool.sendTestMessage.success", getClassName(), MESSAGE_PATH);
            } catch (Exception e) {
                LogUtil.error(this.getClassName(), e, "Fail to send Test Message to Slack");
                message = AppPluginUtil.getMessage("SlackWebhookTool.sendTestMessage.fail", getClassName(), MESSAGE_PATH) + "\n" + StringEscapeUtils.escapeJavaScript(e.getMessage());
            }
            try {
                JSONObject jsonObject = new JSONObject();
                jsonObject.accumulate("message", message);
                jsonObject.write(response.getWriter());
            } catch (Exception e) {
                //ignore
            }
        } else {
            response.setStatus(HttpServletResponse.SC_NO_CONTENT);
        }
    }
    
    protected SlackApi getApi() {
        if (api == null) {
            api = new SlackApi(getPropertyString("url"));
        }
        return api;
    }
    
    protected String getSlackUsername(String username, WorkflowAssignment assignment) {
        String syntax = getPropertyString("usernameTransform");
        syntax = syntax.replaceAll(StringUtil.escapeRegex("{username}"), StringUtil.escapeRegex(username));
        return AppUtil.processHashVariable(syntax, assignment, null, null);
    }
    
    protected void sendMessage(Set<String> channels) {
        SlackMessage message = createMessage();
        if (channels != null && !channels.isEmpty()) {
            for (String channel : channels) {
                if (!channel.isEmpty()) {
                    sendMessage(channel, message);
                }
            }
        } else {
            sendMessage(null, message);
        }
    }
    
    protected void sendMessage(String channel, SlackMessage message) {
        if (channel != null && !channel.isEmpty()) {
            message.setChannel(channel);
        }
        
        getApi().call(message);
    }
    
    protected SlackMessage createMessage() {
        SlackMessage message = new SlackMessage(getPropertyString("text"));
        
        String username = getPropertyString("username");
        if (!username.isEmpty()) {
            message.setUsername(username);
        }
        
        String customIcon = getPropertyString("customIcon");
        if (!customIcon.isEmpty()) {
            if ("joget".equals(customIcon)) {
                HttpServletRequest request = WorkflowUtil.getHttpServletRequest();
                if (request != null) {
                    String url = request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + request.getContextPath() + "/images/v3/logo.png";
                    message.setIcon(url);
                }
            } else if ("url".equals(customIcon)) {
                message.setIcon(getPropertyString("iconUrl"));
            } else {
                message.setIcon(getPropertyString("iconEmoji"));
            }
        }
        
        message.setUnfurlLinks("true".equalsIgnoreCase(getPropertyString("unfurl_links")));
        message.setUnfurlMedia("true".equalsIgnoreCase(getPropertyString("unfurl_media")));
        
        boolean hasAttachment = false;
        SlackAttachment atatchment = new SlackAttachment();
        
        String fallback = getPropertyString("fallback");
        if (!fallback.isEmpty()) {
            atatchment.setFallback(fallback);
        }
        
        String color = getPropertyString("color");
        if (!color.isEmpty()) {
            atatchment.setColor(color);
        }
        
        String pretext = getPropertyString("pretext");
        if (!pretext.isEmpty()) {
            atatchment.setPretext(pretext);
            hasAttachment = true;
        }
        
        String author_name = getPropertyString("author_name");
        if (!author_name.isEmpty()) {
            atatchment.setAuthorName(author_name);
        }
        
        String author_link = getPropertyString("author_link");
        if (!author_link.isEmpty()) {
            atatchment.setAuthorLink(author_link);
        }
        
        String author_icon = getPropertyString("author_icon");
        if (!author_icon.isEmpty()) {
            atatchment.setAuthorIcon(author_icon);
        }
        
        String attachment_title = getPropertyString("attachment_title");
        if (!attachment_title.isEmpty()) {
            atatchment.setTitle(attachment_title);
            hasAttachment = true;
        }
        
        String attachment_title_link = getPropertyString("attachment_title_link");
        if (!attachment_title_link.isEmpty()) {
            atatchment.setTitleLink(attachment_title_link);
        }
        
        String attachment_text = getPropertyString("attachment_text");
        if (!attachment_text.isEmpty()) {
            atatchment.setText(attachment_text);
            hasAttachment = true;
        }
        
        String image_url = getPropertyString("image_url");
        if (!image_url.isEmpty()) {
            atatchment.setImageUrl(image_url);
            hasAttachment = true;
        }
        
        String thumb_url = getPropertyString("thumb_url");
        if (!thumb_url.isEmpty()) {
            atatchment.setThumbUrl(thumb_url);
            hasAttachment = true;
        }
        
        Object[] fields = null;
        Object fieldsProperty = getProperty("fields");
        if (fieldsProperty != null && fieldsProperty instanceof Object[]){
            fields = (Object[]) fieldsProperty;
        }
        if (fields != null && fields.length > 0) {
            for (Object o : fields) {
                Map mapping = (HashMap) o;
                String title = mapping.get("title").toString();
                String value = mapping.get("value").toString();
                String isShort = mapping.get("short").toString();

                SlackField field = new SlackField();
                field.setTitle(title);
                field.setValue(value);
                field.setShorten("true".equalsIgnoreCase(isShort));
                
                atatchment.addFields(field);
                hasAttachment = true;
            }
        }
        
        if (hasAttachment) {
            message.addAttachments(atatchment);
        }
        
        return message;
    }
}
